package ru.ifmo.lift;

import com.evelopers.unimod.runtime.*;
import com.evelopers.unimod.runtime.context.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * 
 * Represents main form of application.
 *
 */
public class MainForm extends JFrame implements ControlledObject 
{
	private static final long serialVersionUID = 0;
	
	public static final int POS_X = 400;
	public static final int POS_Y = 150;
	public static final int WIDTH = 400;
	public static final int HEIGHT = 750;
	
	public static final Color LEFT_PANEL_COLOR = new Color( 240, 230, 220 );
	public static final Color RIGHT_PANEL_COLOR = new Color( 240, 230, 220 );
	public static final Color CENTRAL_PANEL_COLOR = Color.BLACK;
	public static final Color BOTTOM_PANEL_COLOR = new Color( 240, 230, 220 );
	public static final Color OUTER_UPPER_BUTTON_COLOR = new Color( 128, 255, 128 );
	public static final Color OUTER_LOWER_BUTTON_COLOR = new Color( 128, 128, 255 );
	public static final Color OUTER_BUTTON_COLOR2 = Color.WHITE;
	public static final Color INNER_BUTTON_COLOR = new Color( 255, 255, 128 );
	public static final Color INNER_BUTTON_COLOR2 = Color.WHITE;
	public static final Color HIGHLIGHTED_COLOR = Color.ORANGE;
	public static final Color TEXT_COLOR = Color.BLACK;
	public static final Color INFO_TEXT_COLOR = Color.BLACK;
	public static final Color TEXT_AREA_COLOR = Color.WHITE;
	
	public static final String FONT_NAME = "ARIAL";
	
	private EventManager eventManager;
	private Lift lift;
	
	JLabel inControlLabel;
	JLabel outControlLabel;
	JLabel stateLabel;
	
	private JPanel liftPanel;
	private JPanel inButtonsPanel;
	private JPanel outButtonsPanel;
	private JPanel statePanel;

	private Rectangle liftRect;
	private Rectangle insideCtrlRect;
	private Rectangle outwardCtrlRect;
	private Rectangle stateRect;
	
	private JTextArea textArea;
	private JScrollPane scrollPane;
	private JButton btnExit;
	
	private CallButton[] outCallButtons;
	private CallButton[] inCallButtons;
		
	/**
	 * Default constructor.
	 *
	 */
	public MainForm()
	{
		super( "SmartLift" );
		
		insideCtrlRect = new Rectangle( 0, 0, (int)( WIDTH * 0.2 ), (int)( HEIGHT * 0.8 ) );
		liftRect = new Rectangle( (int)( WIDTH * 0.2 ), 0, (int)( WIDTH * 0.58 ), (int)( HEIGHT * 0.8 ) );
		outwardCtrlRect = new Rectangle( (int)( WIDTH * 0.78 ), 0, (int)( WIDTH * 0.22 ), (int)( HEIGHT * 0.8 ) );
		stateRect = new Rectangle( 0, (int)( HEIGHT * 0.8 ), (int)( WIDTH * 0.985 ), (int)( HEIGHT * 0.16 ) );
		
		this.setLocation( POS_X, POS_Y );
		this.setSize( WIDTH, HEIGHT );
		this.setResizable( false );
		this.setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
		
		initializeComponents();
		eventSubscription();
		additionalInitializations();
	}
	
	/**
	 * Closes the form.
	 * @param context Context
	 * @unimod.action.descr  .
	 */
	public void closeForm( StateMachineContext context )
	{
		this.dispose();
	}
	
	/**
	 * Updates the lift buttons state.
	 * @param context Context
	 * @unimod.action.descr   .
	 */
	public void updateButtons( StateMachineContext context ) 
	{
		drawInsideControls();
		drawOutwardControls();
	}
	
	/**
	 * Shows auxillary window with lift breakage information.
	 * @param context
	 * @unimod.action.descr    .
	 */
	public void showBreakageMsg( StateMachineContext context ) 
	{
		BreakageForm form = new BreakageForm( eventManager );
		form.setVisible( true );
	}
	
	/**
	 * Obrains lift object.
	 * @param l Lift
	 */
	public void setLift( Lift l )
	{
		lift = l;
	}
	
	/**
	 * Obtains event manager.
	 * @param manager Event manager
	 */
	public void setEventManager( EventManager manager )
	{
		eventManager = manager;
	}
	
	/**
	 * Returns graphics where lift will be drawn.
	 * @return Graphics
	 */
	public Graphics getLiftGraphics()
	{
		return liftPanel.getGraphics();
	}
	
	/**
	 * Returns rectangle where lift will be drawn.
	 * @return Rectangle
	 */
	public Rectangle getLiftRectangle()
	{
		Rectangle rect = liftPanel.getBounds();
		rect.x = 3;
		rect.y = 10;
		rect.width -= 6;
		rect.height -= 15;
		return rect;
	}
	
	/**
	 * Prints info to the info control.
	 * @param s String for printing
	 */
	public void printInfo( String s )
	{
		textArea.append( s + '\n' );
		JScrollBar scrollBar = scrollPane.getVerticalScrollBar(); 
		scrollBar.setValue( scrollBar.getMaximum() - 1 );
	}
	
	/**
	 * Overrides standart frame paint method.
	 */
	public void paint( Graphics g )
	{
		super.paint( g );
		drawInsideControls();
		drawOutwardControls();
		drawLift();
	}
	
	private void initializeComponents()
	{
		this.getContentPane().setLayout( null );

		//lift panel
		liftPanel = new JPanel( true )
		{
			private static final long serialVersionUID = 0;
			public void paint( Graphics g )
			{
				super.paint( g );
				MainForm.this.drawLift();
			}
		};
		liftPanel.setBackground( CENTRAL_PANEL_COLOR );
		liftPanel.setBounds( liftRect );
		this.getContentPane().add( liftPanel );
		
		//inner control
		inControlLabel = new JLabel( "" );
		inControlLabel.setFont( new Font( FONT_NAME, Font.BOLD, 16 ) );
		inControlLabel.setHorizontalAlignment( JLabel.CENTER );
		inButtonsPanel = new JPanel( new BorderLayout(), true )
		{
			private static final long serialVersionUID = 0;
			public void paint( Graphics g )
			{
				super.paint( g );
				MainForm.this.drawInsideControls();
			}
		};
		inButtonsPanel.setBackground( LEFT_PANEL_COLOR );
		inButtonsPanel.add( inControlLabel, BorderLayout.NORTH );
		inButtonsPanel.setBounds( insideCtrlRect );		
		this.getContentPane().add( inButtonsPanel );
		
		//outward control
		outControlLabel = new JLabel( "" );
		outControlLabel.setFont( new Font( FONT_NAME, Font.BOLD, 16 ) );
		outControlLabel.setHorizontalAlignment( JLabel.CENTER );
		outButtonsPanel = new JPanel( new BorderLayout(), true )
		{
			private static final long serialVersionUID = 0;
			public void paint( Graphics g )
			{
				super.paint( g );
				MainForm.this.drawOutwardControls();
			}
		};
		outButtonsPanel.setBackground( RIGHT_PANEL_COLOR );
		outButtonsPanel.add( outControlLabel, BorderLayout.NORTH );
		outButtonsPanel.setBounds( outwardCtrlRect );
		this.getContentPane().add( outButtonsPanel );
		
		//status panel
		statePanel = new JPanel( new BorderLayout() );
		statePanel.setBackground( BOTTOM_PANEL_COLOR );
		statePanel.setBounds( stateRect );
		this.getContentPane().add( statePanel );
		
		//status label
		stateLabel = new JLabel( " :" );
		statePanel.add( stateLabel, BorderLayout.NORTH );

		//text area
		textArea = new JTextArea();
        textArea.setEditable( false );
        textArea.setBackground( TEXT_AREA_COLOR );
        textArea.setForeground( INFO_TEXT_COLOR );
        textArea.setSelectionColor( new Color( (int)(Math.pow(2, 23) - 1) ^ INFO_TEXT_COLOR.getRGB() ) );

        //scroll pane
        scrollPane = new JScrollPane( textArea );
        scrollPane.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBackground( BOTTOM_PANEL_COLOR );
        statePanel.add( scrollPane, BorderLayout.CENTER );
        
		//exit button
		btnExit = new JButton( "" );
		btnExit.setBackground( BOTTOM_PANEL_COLOR );
		statePanel.add( btnExit, BorderLayout.EAST );
	}
	
	private void eventSubscription()
	{
		//exit button
		btnExit.addActionListener( new ActionListener(){
			public void actionPerformed( ActionEvent e )
			{
				
				com.evelopers.unimod.core.stateworks.Event event;
				event = new com.evelopers.unimod.core.stateworks.Event( LiftEventProvider.seExit );
				eventManager.handle( event, StateMachineContextImpl.create() );
			}
		} );
		
		//adds mouse listening
		outButtonsPanel.addMouseListener( new MouseAdapter() {
			public void mousePressed( MouseEvent e ) 
			{
				int x = e.getX();
				int y = e.getY();
				for ( int i = 0; i < outCallButtons.length; i++ )
				{
					CallButton outBtn = outCallButtons[i];
					Rectangle rect = outBtn.rect;
					if ( ( x >= rect.x ) && ( x <= rect.x + rect.width ) && 
					 	 ( y >= rect.y ) && ( y <= rect.y + rect.height) )
					{
						lift.addNewTask( outBtn.floor, outBtn.direction, true );
						MainForm.this.repaint();
						break;
					}
				}
			}
		} );
				
		inButtonsPanel.addMouseListener( new MouseAdapter() {
			public void mousePressed( MouseEvent e ) 
			{
				int x = e.getX();
				int y = e.getY();
				for ( int i = 0; i < inCallButtons.length; i++ )
				{
					CallButton inBtn = inCallButtons[i];
					Rectangle rect = inBtn.rect;
					int x0 = rect.x + rect.width / 2;
					int y0 = rect.y + rect.height / 2; 
					int r = inBtn.radius;
					
					if ( Math.pow( x - x0, 2 ) + Math.pow( y - y0, 2 ) < Math.pow( r, 2 ) )
					{
						lift.addNewTask( inBtn.floor, inBtn.direction, false );
						MainForm.this.repaint();
						break;
					}
						
				}
			}
		} );	
	}
	
	private void additionalInitializations()
	{
		outCallButtons = new CallButton[ 2 * Lift.FLOORS_COUNT - 2 ];
		inCallButtons = new CallButton[ Lift.FLOORS_COUNT ];
		
		Rectangle outwardCtrlRect = outButtonsPanel.getBounds();
		outwardCtrlRect.x = 0;
		outwardCtrlRect.y = 0;
		
		int y0 = (int)( outwardCtrlRect.y + outwardCtrlRect.height * 0.05 );  
		int dy = (int)( outwardCtrlRect.height * 0.9 / ( Lift.FLOORS_COUNT - 1 ) / 2 );
		for ( int floor = 0; floor < Lift.FLOORS_COUNT; floor++ )
		{
			Rectangle rect;

			rect = new Rectangle( outwardCtrlRect.x, y0 + dy * ( floor * 2 - 1 ), outwardCtrlRect.width, dy );
			if ( floor > 0  )
			{
				Rectangle upperRect = new Rectangle();
				upperRect.x = (int)( rect.x + rect.width * 0.2 ); 
				upperRect.y = (int)( rect.y + rect.height * 0.2 ); 
				upperRect.width = (int)( rect.width * 0.6 );
				upperRect.height = (int)( rect.height * 0.75 );
				outCallButtons[ floor * 2 - 1 ] = new CallButton( Lift.FLOORS_COUNT - floor, Lift.UPWARD, upperRect );
			}

			rect = new Rectangle( outwardCtrlRect.x, y0 + dy * 2 * floor, outwardCtrlRect.width, dy );
			if ( floor < Lift.FLOORS_COUNT - 1 )
			{
				Rectangle lowerRect;
				lowerRect = new Rectangle();
				lowerRect.x = (int)( rect.x + rect.width * 0.2 ); 
				lowerRect.y = (int)( rect.y ); 
				lowerRect.width = (int)( rect.width * 0.6 );
				lowerRect.height = (int)( rect.height * 0.75 );
				outCallButtons[ floor * 2 ] = new CallButton( Lift.FLOORS_COUNT - floor, Lift.DOWNWARD, lowerRect );
			}
		}
		
		Rectangle insideCtrlRect = inButtonsPanel.getBounds();
		insideCtrlRect.x = 0;
		insideCtrlRect.y = 0;

		y0 = (int)( insideCtrlRect.y + insideCtrlRect.height * 0.05 );  
		dy = (int)( insideCtrlRect.height * 0.9 / Lift.FLOORS_COUNT );
		for ( int floor = 0; floor < Lift.FLOORS_COUNT; floor++ )
		{
			Rectangle rect;
			rect = new Rectangle( insideCtrlRect.x, y0 + dy * floor, insideCtrlRect.width, dy ); 
			inCallButtons[ floor ] = new CallButton( Lift.FLOORS_COUNT - floor, Lift.UNKNOWN, rect );
		}
	}
	
	private void drawInsideControls()
	{
		Graphics g = inButtonsPanel.getGraphics();
		g.setFont( new Font( FONT_NAME, Font.BOLD, 16 ) );
		
		for ( int k = 0; k < inCallButtons.length; k++ )
		{
			CallButton btn = inCallButtons[ k ];
			Rectangle rect = btn.rect;
			int x0 = rect.x + rect.width / 2;
			int y0 = rect.y + rect.height / 2;
			rect.x = x0 - btn.radius;
			rect.y = y0 - btn.radius;
			rect.width = 2 * btn.radius;
			rect.height = 2 * btn.radius;

			//button
			g.setColor( INNER_BUTTON_COLOR );
			g.fillOval( rect.x, rect.y, rect.width, rect.height );
			g.setColor( Color.black );
			g.drawOval( rect.x, rect.y, rect.width, rect.height );

			if ( lift.isButtonHighlighted( btn.floor, Lift.UNKNOWN, false ) )
				g.setColor( HIGHLIGHTED_COLOR );
			else					
				g.setColor( INNER_BUTTON_COLOR2 );
			g.fillOval( rect.x + 8, rect.y + 8, rect.width - 16, rect.height - 16 );
			g.setColor( TEXT_COLOR );
			g.drawOval( rect.x + 8, rect.y + 8, rect.width - 16, rect.height - 16 );
			String str = String.valueOf( btn.floor );
			g.drawString( str, rect.x + rect.width / 2 - 4, rect.y + rect.height / 2 + 6 );
		}
	}
	
	private void drawOutwardControls()
	{
		Graphics g = outButtonsPanel.getGraphics();
		g.setFont( new Font( FONT_NAME, Font.BOLD, 16 ) );

		for ( int k = 0; k < outCallButtons.length; k++ )
		{
			Rectangle rect;
			CallButton btn = outCallButtons[k];

			//lower button
			if ( btn.getDirection() == Lift.DOWNWARD )
			{
				rect = btn.rect;
				g.setColor( OUTER_LOWER_BUTTON_COLOR );
				g.fillRect( rect.x, rect.y, rect.width, rect.height );
				g.setColor( TEXT_COLOR );
				g.drawRect( rect.x, rect.y, rect.width, rect.height );
				if ( lift.isButtonHighlighted( btn.floor, btn.direction, true ) )
					g.setColor( HIGHLIGHTED_COLOR );
				else					
					g.setColor( OUTER_BUTTON_COLOR2 );
				
				int[] x = new int[3];
				int[] y = new int[3];
				x[0] = rect.x + 5; 
				y[0] = rect.y + 10;
				x[1] = rect.x + rect.width - 5;
				y[1] = rect.y + 10;
				x[2] = rect.x + rect.width / 2;
				y[2] = rect.y + rect.height - 5;
				
				g.fillPolygon( x, y, 3 );
				g.setColor( TEXT_COLOR );
				g.drawPolygon( x, y, 3 );
				g.drawString( String.valueOf( btn.floor ), rect.x + rect.width / 2 - 4, rect.y + rect.height / 2 + 6 );
			}
			else
			{
				//upper button
				rect = btn.rect;
				g.setColor( OUTER_UPPER_BUTTON_COLOR );
				g.fillRect( rect.x, rect.y, rect.width, rect.height );
				g.setColor( TEXT_COLOR );
				g.drawRect( rect.x, rect.y, rect.width, rect.height );
				if ( lift.isButtonHighlighted( btn.floor, btn.direction, true ) )
					g.setColor( HIGHLIGHTED_COLOR );
				else					
					g.setColor( OUTER_BUTTON_COLOR2 );
				
				int[] x = new int[3];
				int[] y = new int[3];
				x[0] = rect.x + 5; 
				y[0] = rect.y + rect.height - 10;
				x[1] = rect.x + rect.width - 5;
				y[1] = rect.y + rect.height - 10;
				x[2] = rect.x + rect.width / 2;
				y[2] = rect.y + 5;
				
				g.fillPolygon( x, y, 3 );
				g.setColor( TEXT_COLOR );
				g.drawPolygon( x, y, 3 );
				g.drawString( String.valueOf( btn.floor ), rect.x + rect.width / 2 - 4, rect.y + rect.height / 2 + 6 );
			}
		}
	}
	
	private void drawLift()
	{
		if ( lift != null )
		{
			lift.drawLift();
		}
	}
}

/**
 * 
 * Lift button custom class. 
 *
 */
final class CallButton {
	public int direction;
	public int floor;
	public int radius;

	public Rectangle rect;
	
	/**
	 * Constructor.
	 * @param f Floor number
	 * @param dir Direction
	 * @param r Rectangle for button painting
	 */
	public CallButton( int f, int dir, Rectangle r )
	{
		floor = f;
		direction = dir;
		rect = r;
		radius = Math.min( rect.width, rect.height ) / 3;
	}
	
	/**
	 * Returns direction what this button points on. 
	 */
	public int getDirection()
	{
		return direction;
	}
}