package ru.ifmo.lift;

import java.util.*;

/**
 * 
 * List of the tasks for a lift.
 *
 */
public class TaskList {
	private ArrayList array;
	
	private int minValue = Integer.MAX_VALUE;
	private int maxValue = Integer.MIN_VALUE;
	
	/**
	 * Constructor.
	 *
	 */
	TaskList()
	{
		array = new ArrayList();
	}
	
	/**
	 * Returns minimal floor number in the list. 
	 */
	public int minElem()
	{
		return minValue;
	}
	
	/**
	 * Returns maximal floor number in the list.
	 */
	public int maxElem()
	{
		return maxValue;
	}
	
	/**
	 * Adds new task to list.
	 * @param i Floor number
	 * @param outCall equal to true if it's outer call and to false otherwise 
	 */
	public void add( int i, boolean outCall )
	{
		array.add( new Task( i, outCall) );
		if ( i < minValue )
			minValue = i;
		if ( i > maxValue )
			maxValue = i;
	}
	
	/**
	 * Removes task with specified floor number from the list
	 */
	public void removeTask( int iTask )
	{
		ArrayList tempArray = new ArrayList();
		for ( int i = 0; i < array.size(); i++ )
			if ( ( ( Task )array.get( i ) ).floor != iTask )
				tempArray.add( array.get( i ) );

		array = tempArray;
		
		definesMinMax();
	}
	
	/**
	 * Defines either the list contains specified task or not. 
	 */
	public boolean contains( int floor, boolean outCall )
	{
		for ( int i = 0; i < array.size(); i++ )
		{
			Task t = (Task)array.get( i ); 
			if ( ( t.floor == floor ) && ( t.outCall == outCall ) )
				return true;
		}
		return false;
	}
	
	/**
	 * Returns true if list is empty and false otherwise. 
	 */
	public boolean isEmpty()
	{
		return array.isEmpty();
	}
	
	/**
	 * Returns number of the tasks in the list. 
	 */
	public int size()
	{
		return array.size();
	}
	
	/**
	 * Returns i-th task. 
	 */
	public int itemFloor( int i )
	{
		if ( ( i < 0 ) || ( i >= array.size() ) )
			return 0;
		return ( (Task)array.get( i ) ).floor;
	}
	
	/**
	 * Returns outCall flag of i-th task. 
	 */
	public boolean itemOutCall( int i )
	{
		if ( ( i < 0 ) || ( i >= array.size() ) )
			return false;
		return ( (Task)array.get( i ) ).outCall;
	}
	
	private void definesMinMax()
	{
		int max = Integer.MIN_VALUE;
		int min = Integer.MAX_VALUE;
		for ( int i = 0; i < array.size(); i++ )
		{
			if ( ( ( Task )array.get( i ) ).floor > max )
				max = ( ( Task )array.get( i ) ).floor;
			if ( ( ( Task )array.get( i ) ).floor < min )
				min = ( ( Task )array.get( i ) ).floor;
		}
		minValue = min;
		maxValue = max;
	}
}

/**
 * 
 * Task class.
 *
 */
final class Task
{
	public int floor;
	public boolean outCall;
	
	/**
	 * Constructor.
	 * @param f Floor number
	 * @param out outCall flag
	 */
	public Task( int f, boolean out )
	{
		floor = f;
		outCall = out;
	}
}